#!/usr/bin/perl

# Upgrade database contents from version 1.1.3 (and 1.0.6) to 1.1.4
# Copyright (c) 2002-2003 Josef Zlomek
#
# This file is part of Fast File Search (FFS).
#
# FFS is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA,
# or download it from http://www.gnu.org/licenses/gpl.html

use strict;
use DBI;
use Getopt::Long qw(:config bundling no_ignore_case);
use String::CRC32;

# include the common package
$_ = $0;
$_ =~ s/[^\/]*$/bin\/common.pl/;
require;

my ($help);
GetOptions ('help|h' => \$help,
		);
if ($help)
{   
	&help ();
	exit 0;
}

my %c;
my ($db, $res);

&read_config (\%c);
&db_connect (\%c, undef, \$db);
exit 1 if (!$db);

$db->do ("ALTER TABLE path ADD PPID INT UNSIGNED NOT NULL");
$db->do ("ALTER TABLE path ADD KEY(PPID)");

$res = $db->selectall_arrayref ("SELECT HID FROM host WHERE HostType=$c{SMB}");
foreach (@$res)
{
	my $HID = $_->[0];
	my $hash = crc32 ("", crc32 (pack ("II", $HID, 0)));
	my $PID = $db->selectrow_array ("SELECT PID FROM path WHERE HID=$HID AND SID=0 AND Hash=$hash AND FullName=''");
	if (!$PID)
	{
		$db->do ("INSERT INTO path (Hash,PPID,HID,SID,ShortName,FullName) VALUES ($hash,0,$HID,0,'','')");
	}
}

$res = $db->selectall_arrayref ("SELECT PID,HID,SID,FullName FROM path");
foreach (@$res)
{
	my $PID = $_->[0];
	my $HID = $_->[1];
	my $SID = $_->[2];
	my $path = $_->[3];

	if ($path eq '')
	{
		$db->do ("UPDATE path SET PPID=0 WHERE PID=$PID");
	}
	else
	{
		$path =~ s/\/*[^\/]*$//;
		$SID = 0 if ($path eq '');
		my $hash = crc32 ($path, crc32 (pack ("II", $HID, $SID)));
		my $PPID = $db->selectrow_array ("SELECT PID FROM path WHERE HID=$HID AND SID=$SID AND Hash=$hash AND FullName=" . $db->quote ($path));
		$PPID = 0 if (!defined ($PPID));
		$db->do ("UPDATE path SET PPID=$PPID WHERE PID=$PID");
	}
}

&db_disconnect ($db);

sub help
{
	print "Fast File Search - upgrade database contents to version 1.1.4\n";
	print "Usage: upgrade-1.1.3-1.1.4.pl [OPTION]...\n\n";
	print "    -c, --common, --master   Use common (master) database (default)\n";
	print "    -p, --private, --slave   Use private (slave) database\n";
	print "    -h, --help               Display this help and exit\n\n";
}
